#include "soundtypes.h"

#include <vector>

class Stream
{
private:
	bool bit16;
	bool stereo;
	unsigned herz;
	
private:
	/* Reads 16-bit mono entries */
	static unsigned read16(stent *dest, unsigned count);
	/* Reads 8-bit stereo entries */
	static unsigned readste(stent *dest, unsigned count);
	/* Reads 8-bit mono entries */
	static unsigned read8(stent *dest, unsigned count);
	/* Writes 16-bit mono entries */
	static void write16(const stent *src, unsigned count);
	/* Writes 8-bit stereo entries */
	static void writeste(const stent *src, unsigned count);
	/* Writes 8-bit mono entries */
	static void write8(const stent *src, unsigned count);
	/* 16-bit stereo reading/writing does not need conversion. */
	
public:
	Stream(const char *settings);

	inline const unsigned rate() const { return herz; }
	
	unsigned Read(stent *dest, unsigned count) const;
    void Write(const stent *dest, unsigned count) const;
    
    void Flush() const;
    
    static int SafeRead(uint8* target, unsigned nbytes, unsigned align);
};
